#!/usr/bin/perl -w

# Script to convert a PSS excel file to an EPICS .db file
# using a template file for each excel worksheet.

use strict;
use Spreadsheet::ParseExcel;
use File::Basename;

# Convert excel worksheet name to template file ID.
my %sheetID=();
$sheetID{'Analog'}    ='ANALOG';
$sheetID{'IO'}        ='IO';
$sheetID{'Beamlie|PLC Status'}='PLC';
$sheetID{'Warning Bits|PLC Bits'}='WARNING_PLC_BITS';
$sheetID{'Fault|Trip Bits'}='FTBITS';
$sheetID{'FIFOs'}='FTW';

# Output file name
my $outfile = $ARGV[0];
shift @ARGV;

# Excel file 
my $excelfile = $ARGV[0];
shift @ARGV;

# Directory with Excel and template files
my $xlsdir=dirname($excelfile);

# Read template files
#my $templatedir="../";
#my $templatefile=$templatedir.$file;
#my @lines;
my %templateArray;
my $line;
my ($id, $file);

# Create depends output file ----------
my $dependsfile=$outfile.'.depends';
open(OUTFILE, "> $dependsfile") or die "Cannot create $dependsfile\n";
print OUTFILE "$outfile: $excelfile ../Makefile\n";
#print "Depends file name is $outfile\n";

# Loop over command line arguments
my $dfile;
my %dependsHash=();
for my $arg ( @ARGV ) {
  ($id,$file)=split(/=/,$arg,2);
  next if !$file;
  my @lines=();
  $dfile=$file;
  if (!open(INFILE, "< $file")) {
    $dfile="../$file";
    if (!open(INFILE, "< ../$file")) {
      $dfile="$xlsdir/$file";
      open(INFILE, "< $xlsdir/$file") or die "Cannot find or open template file $file: $!";
    }
  }
  # Read the template file and put data into and array
  @lines= <INFILE>;
  $templateArray{$id}=\@lines;

  # Add a dependancy line to the depends output file
  if (!$dependsHash{$dfile}) {
    $dependsHash{$dfile}=1;
    print OUTFILE "$outfile: $dfile\n";
  }
  close(INFILE);
}
close(OUTFILE);


# Print out template information ----------
#foreach my $key (keys %templateArray) {
#  my @lines=@{$templateArray{$key}};
#  print "$key --------------------------------------\n";
#  foreach $line (@lines) { 
#    print $line;
#  }      
#}      

# Print file file name information ----------
print "\nCreating $outfile from excel file $excelfile \n";
print "Using template files @ARGV\n\n";

  # Read excel file -----------
  my $parser   = Spreadsheet::ParseExcel->new();
  my $workbook = $parser->parse($excelfile);
  die $parser->error(), ".\n" if !defined $workbook;

  # Open output file ----------
  open(OUT, "> $outfile") or die "Cannot create $outfile\n";

  # Loop over excel worksheets -----------
  for my $worksheet ( $workbook->worksheets() ) {

    my $sheetName = $worksheet->get_name();
    my $sheetType=$sheetID{$sheetName};

    # Skip unknown worksheet types
    next if ! $sheetType;

    my ( $row_min, $row_max ) = $worksheet->row_range();
    my ( $col_min, $col_max ) = $worksheet->col_range();
    #print "row_min=$row_min row_max=$row_max\n";
    #print "col_min=$col_min col_max=$col_max\n";

    # Loop over the two pss types -----------
    for my $pssType ( ( "PA_", "PB_" ) ) {
    
      my $key=$pssType.$sheetType;
      next if ! $templateArray{$key};
      #print "Processing $key\n";
      my @lines=@{$templateArray{$key}};

      print "No template for $pssType.$sheetType\n" if ! @lines;
      next if !@lines;

      # Loop over rows in excel worksheet -----------
      my ($used, $type, $name, $dcmAddr, $fanAddr, $desc, $aslo, $prec );
      for my $row ( $row_min .. $row_max ) {
        next if $row==0;
        next if !$worksheet->get_cell($row, 0);

        $used    = $worksheet->get_cell($row, 0)->value();
        next if $used eq "";
        next if $used ne "A" && $used ne "B"  && $used ne "X";
        next if $used eq "A" && $pssType eq "PB_";
        next if $used eq "B" && $pssType eq "PA_";

        $type    = $worksheet->get_cell($row, 1)->value();
        $name    = $worksheet->get_cell($row, 2)->value();
        $dcmAddr = $worksheet->get_cell($row, 4)->value() if $worksheet->get_cell($row, 4);
        $fanAddr = $worksheet->get_cell($row, 6)->value() if $worksheet->get_cell($row, 6);
        $desc    = $worksheet->get_cell($row, 7)->value();
        $desc=~ s/\s+$//;  # Remove trailing blanks
        $aslo  = $worksheet->get_cell($row, 8)->value() if  $worksheet->get_cell($row, 8);
        $prec  = $worksheet->get_cell($row, 9)->value() if  $worksheet->get_cell($row, 9);

        # Loop over each line of template file
        my $newline;
        foreach $line (@lines) { 
          $newline = $line; 
          $newline =~s/\$\(name\)/$name/;
          $newline =~s/\$\(desc\)/$desc/;
          $newline =~s/\$\(dcmAddr\)/$dcmAddr/;
          $newline =~s/\$\(prec\)/$prec/ if $prec;
          $newline =~s/\$\(aslo\)/$aslo/ if $aslo;
          $newline =~s/\$\(fanAddr\)/$fanAddr/;
          print OUT $newline;
        }      
      }
    }
  }

#print "\nDone!!\n";

